// 找到要执行的核心文件
// 1) 要解析用户的参数
const program = require('commander');
const chalk = require('chalk');
const path = require('path');
const { version } = require('./constants');

const cleanArgs = (cmd) => {
  const args = {};
  cmd.options.forEach((o) => {
    const key = o.long.slice(2);
    if (cmd[key]) args[key] = cmd[key];
  });
  return args;
};
// vue create projectname
// npm i eslint
// npx eslint --init
// vscode中配置eslint 保存时修复
const mapActions = {
  create: {
    alias: 'c', // 别名
    description: 'create a new project', // 描述信息
    examples: [
      'vepal-cli create <project-name>', // 命令示例
    ],
  },
  config: {
    alias: 'conf',
    description: 'config project variable',
    examples: [
      'vepal-cli config set <k> <v>',
      'vepal-cli config get <k>',
    ],
  },
  '*': {
    alias: '',
    description: 'command not found',
    examples: [],
  },
};
// Object.keys() 循环创建命令
Reflect.ownKeys(mapActions).forEach((action) => {
  if (action === 'create') {
    program
      .command(action)
      .alias(mapActions[action].alias)
      .description(mapActions[action].description)
      .option('-f , --force', 'overwrite target')
      .action((name, cmd) => {
        // console.log(name, cleanArgs(cmd));
        require(path.resolve(__dirname, action))(...process.argv.slice(3), cleanArgs(cmd));
      });
  } else {
    program
      .command(action) // 配置命令的名字
      .alias(mapActions[action].alias) // 命令的别名
      .description(mapActions[action].description)// 命令对应的描述
      .action((name, cmd) => {
        if (action === '*') { // 访问不到对应的命令 就打印找不到命令
          console.log(mapActions[action].description);
        } else { // create config ....
          // console.log(action); // create / config
          // vepal-cli create xxx // [node,vepal-cli,create,xxx]
          require(path.resolve(__dirname, action))(...process.argv.slice(3));
        }
      });
  }
});
// 监听用户的help 事件
program.on('--help', () => {
  console.log('\nExamples:');
  Reflect.ownKeys(mapActions).forEach((action) => {
    mapActions[action].examples.forEach((example) => {
      console.log(`  ${example}`);
    });
  });
  console.log();
  console.log(`Run ${chalk.cyan('vepal-cli <command> --help')} show details`);
});

// 解析用户传递过来的参数
// process.argv 为用户命令行传递过来的参数
// version 当前cli项目的版本号
program.version(version).usage('<command> [option]');
program.parse(process.argv);
